<?php
/*
Plugin Name: Widget Inclusion for ManyChat
Plugin URI:  https://botprophet.com
Description: This plugin adds the ManyChat Core Javascript Snippet, plus sets a time availability for your widgets 
Version:     2.0
Author:      SocialSource.nl | Paul de Lange
Author URI:  https://socialsource.nl
License:     GPL2
License URI: https://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html

Copyright 2019 (email : paul@chatbotdojo.com)
ManyChat for Wordpress is free software: but you can NOT redistribute it as your own.
 
BotProphet ManyChat Inclusion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with (MC Wordpress). If not, see https://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
*/

class Botprophet_ManyChat_TimeZone {
	
	public static function get_current_timezone (){
		$current_offset  = get_option('gmt_offset');
		$tzstring        = get_option('timezone_string');
		$check_zone_info = true;
		if ( false !== strpos($tzstring,'Etc/GMT') )
			$tzstring = '';
		if ( empty($tzstring) ) { // Create a UTC+- zone if no timezone string exists
			$check_zone_info = false;
			if ( 0 == $current_offset )
				$tzstring = 'UTC+0';
			elseif ($current_offset < 0)
				$tzstring = 'UTC' . $current_offset;
			else
				$tzstring = 'UTC+' . $current_offset;
		}
		return date_default_timezone_set($tzstring);
	}
	
}


class Botprophet_ManyChat_Widgets_Backend {
	
	// Set Timezone Class
	private $tmzn;
		
    public function __construct() {
		
		// Set Timezone Instance
		$this->tmzn = new Botprophet_ManyChat_TimeZone;
		
    	// Hook into the admin menu
    	add_action( 'admin_menu', array( $this, 'create_plugin_settings_page' ) );
        // Add Settings and Fields
    	add_action( 'admin_init', array( $this, 'setup_sections' ) );
    	add_action( 'admin_init', array( $this, 'setup_fields' ) );
    }
	
    public function create_plugin_settings_page() {
    	// Add the menu item and page
    	$page_title = 'ManyChat Settings Page';
    	$menu_title = 'ManyChat Plugin';
    	$capability = 'manage_options';
    	$slug       = 'bp_manychat_fields';
    	$callback   = array( $this, 'plugin_settings_page_content' );
    	$icon       = 'dashicons-admin-plugins';
    	$position   = 100;
    	//add_menu_page( $page_title, $menu_title, $capability, $slug, $callback, $icon, $position );
		add_submenu_page( 'options-general.php', $page_title, $menu_title, $capability, $slug, $callback );
    }
	
    public function plugin_settings_page_content() {
	?>
    	<div class="wrap">
    		<h2>ManyChat Settings Page</h2><?php
            if ( isset( $_GET['settings-updated'] ) && $_GET['settings-updated'] ){
				  // May be double loaded
                  // $this->admin_notice();
            } ?>
    		<form method="POST" action="options.php">
                <?php
                    settings_fields( 'bp_manychat_fields' );
                    do_settings_sections( 'bp_manychat_fields' );
                    submit_button();
                ?>
    		</form>
    	</div> 
	<?php
    }
    
    public function admin_notice() { 
		?>
        <div class="notice notice-success is-dismissible">
            <p>Your settings have been updated!</p>
        </div>
		<?php
    }
	
    public function setup_sections() {
        add_settings_section( 'manychat_main_include_js', 'General ManyChat Javascript (included in header)', array( $this, 'section_callback' ), 'bp_manychat_fields' );
        // add_settings_section( 'manychat_livechat_widget', 'ManyChat (Live Chat) Widget', array( $this, 'section_callback' ), 'bp_manychat_fields' );
        // add_settings_section( 'our_third_section', 'My Third Section Title', array( $this, 'section_callback' ), 'bp_manychat_fields' );
    }
	
    public function section_callback( $arguments ) {
    	switch( $arguments['id'] ){
    		case 'manychat_main_include_js':
    			echo 'In this section you enter the core ManyChat Javascript!';
    			break;
    		case 'manychat_livechat_widget':
    			echo 'In this section we configure the Live Chat Widget (or another one if you prefer)';
    			break;
    		case 'our_third_section':
    			echo 'Third time is the charm!';
    			break;
    	}
    }
	
    public function setup_fields() {
		
		$this->tmzn->get_current_timezone();

        $fields = array(
			// First section
			array(
        		'uid' => 'bp_manychat_textarea_header_js',
        		'label' => 'Main Javascript Code',
        		'section' => 'manychat_main_include_js',
        		'type' => 'textarea',
        		'supplimental' => '(this will be placed in the &lt;head&gt; tags)'				
        	),	
			array(
        		'uid' => 'bp_manychat_active',
        		'label' => 'Is ManyChat Active?',
        		'section' => 'manychat_main_include_js',
        		'type' => 'radio',
        		'options' => array(
        			'yes' => 'Yes',
        			'no' => 'No',
        		),
                'default' => array()
        	),
			array(
        		'uid' => 'bp_manychat_timeractive',
        		'label' => 'Do you want to use time-based planning?',
        		'section' => 'manychat_main_include_js',
        		'type' => 'radio',
        		'options' => array(
        			'yes' => 'Yes',
        			'no' => 'No',
        		),
                'default' => array()
        	),
        	array(
        		'uid' => 'bp_manychat_start_widget',
        		'label' => 'Start time widget',
        		'section' => 'manychat_main_include_js',
        		'type' => 'text',			
				'placeholder' => 'hh:mm',
				'helper' => 'Please use 24 hour format, like 09:30. Now is ' . current_time('H:i'),
				'supplemental' => 'This is the time your widget starts working',
				'default' => date('G:i')
        	),
        	array(
        		'uid' => 'bp_manychat_stop_widget',
        		'label' => 'End time widget',
        		'section' => 'manychat_main_include_js',
        		'type' => 'text',			
				'placeholder' => 'hh:mm',
				'helper' => 'Please use 24 hour format, like 17:30. Now is ' . current_time('H:i'),
				'supplemental' => 'This is the time your widget stops working',
				'default' => date('G:i')
        	)        	
 //,
			
			// Second section [currently nothing active]
			/*
 			array(
        		'uid' => 'bp_manychat_active_livechat_widget',
        		'label' => 'Is this Widget Active?',
        		'section' => 'manychat_livechat_widget',
        		'type' => 'radio',
        		'options' => array(
        			'yes' => 'Yes',
        			'no' => 'No',
        		),
                'default' => array()
        	),
        	array(
        		'uid' => 'bp_manychat_start_livechat_widget',
        		'label' => 'Start time widget',
        		'section' => 'manychat_livechat_widget',
        		'type' => 'text',			
				'placeholder' => 'hh:mm',
				'helper' => 'Please use 24 hour format, like 09:30. Now is ' . current_time('H:i'),
				'supplemental' => 'This is the time your widget starts working',
				'default' => date('G:i')
        	),
        	array(
        		'uid' => 'bp_manychat_stop_livechat_widget',
        		'label' => 'End time widget',
        		'section' => 'manychat_livechat_widget',
        		'type' => 'text',			
				'placeholder' => 'hh:mm',
				'helper' => 'Please use 24 hour format, like 17:30. Now is ' . current_time('H:i'),
				'supplemental' => 'This is the time your widget stops working',
				'default' => date('G:i')
        	),
        	array(
        		'uid' => 'bp_manychat_textarea_livechat_widget',
        		'label' => 'Widget Javascript Code',
        		'section' => 'manychat_livechat_widget',
        		'type' => 'textarea',
        		'supplimental' => '(this will be placed in the &lt;footer&gt; of the page)'				
        	) 
			*/
			// N section
			
        );
    	foreach( $fields as $field ){
        	add_settings_field( $field['uid'], $field['label'], array( $this, 'field_callback' ), 'bp_manychat_fields', $field['section'], $field );
            register_setting( 'bp_manychat_fields', $field['uid'] );
    	}
    }
	
    public function field_callback( $arguments ) {
        $value = get_option( $arguments['uid'] );
        if( ! $value ) {
            $value = $arguments['default'];
        }
        switch( $arguments['type'] ){
            case 'text':
            case 'password':
            case 'number':
                printf( '<input name="%1$s" id="%1$s" type="%2$s" placeholder="%3$s" value="%4$s" />', $arguments['uid'], $arguments['type'], $arguments['placeholder'], $value );
                break;
            case 'textarea':
                printf( '<textarea name="%1$s" id="%1$s" placeholder="%2$s" rows="5" cols="50">%3$s</textarea>', $arguments['uid'], $arguments['placeholder'], $value );
                break;
            case 'select':
            case 'multiselect':
                if( ! empty ( $arguments['options'] ) && is_array( $arguments['options'] ) ){
                    $attributes = '';
                    $options_markup = '';
                    foreach( $arguments['options'] as $key => $label ){
                        $options_markup .= sprintf( '<option value="%s" %s>%s</option>', $key, selected( $value[ array_search( $key, $value, true ) ], $key, false ), $label );
                    }
                    if( $arguments['type'] === 'multiselect' ){
                        $attributes = ' multiple="multiple" ';
                    }
                    printf( '<select name="%1$s[]" id="%1$s" %2$s>%3$s</select>', $arguments['uid'], $attributes, $options_markup );
                }
                break;
            case 'radio':
            case 'checkbox':
                if( ! empty ( $arguments['options'] ) && is_array( $arguments['options'] ) ){
                    $options_markup = '';
                    $iterator = 0;
                    foreach( $arguments['options'] as $key => $label ){
                        $iterator++;
                        $options_markup .= sprintf( '<label for="%1$s_%6$s"><input id="%1$s_%6$s" name="%1$s[]" type="%2$s" value="%3$s" %4$s /> %5$s</label><br/>', $arguments['uid'], $arguments['type'], $key, checked( $value[ array_search( $key, $value, true ) ], $key, false ), $label, $iterator );
                    }
                    printf( '<fieldset>%s</fieldset>', $options_markup );
                }
                break;
        }
        if( $helper = $arguments['helper'] ){
            printf( '<span class="helper"> %s</span>', $helper );
        }
        if( $supplimental = $arguments['supplimental'] ){
            printf( '<p class="description">%s</p>', $supplimental );
        }
    }
	
}

class Botprophet_ManyChat_Widgets_Frontend {

    public function __construct()
    {
		add_action('wp_head', array( $this, 'hook_mc_main_javascript') );
		// add_action('wp_footer', array( $this, 'manychat_livechat_widget') );		
		// add_filter('the_content', array( $this, 'manychat_livechat_wordpress_test') ); // only or testing
    }
	
	public function isBetween($from, $till, $input) 
	{
		$f = DateTime::createFromFormat('H:i', $from);
		$t = DateTime::createFromFormat('H:i', $till);
		$i = DateTime::createFromFormat('H:i', $input);
		if ($f > $t) $t->modify('+1 day');
		return ($f <= $i && $i <= $t) || ($f <= $i->modify('+1 day') && $i <= $t);
	}
 
	public function manychat_livechat_wordpress_test( $content ) 
	{		
		$this->tmzn->get_current_timezone();

		$current_time = current_time('H:i');
		$open = get_option('bp_manychat_start_livechat_widget'); // "9:30 am";
		$close = get_option('bp_manychat_stop_livechat_widget'); // "5:30 pm";

		$f = DateTime::createFromFormat('H:i', $open);
		$t = DateTime::createFromFormat('H:i', $close);
		$i = DateTime::createFromFormat('H:i', $current_time);
		if ($f > $t) $t->modify('+1 day');
		if (($f <= $i && $i <= $t) || ($f <= $i->modify('+1 day') && $i <= $t)){
			$content .= '<footer>Cool Stuff, hello!<br>' . current_time('d/m/Y H:i') . '<br>' . date('H:i') . '<br>' . $f->format('d/m/Y H:i') . '<br>' . $t->format('d/m/Y H:i') . '<br>' . $i->format('d/m/Y H:i') . '</footer>';
		}else{
			$content .= '<footer>Bad Stuff, hello!<br>' . current_time('d/m/Y H:i') . '<br>' . date('H:i') . '<br>' . $f->format('d/m/Y H:i') . '<br>' . $t->format('d/m/Y H:i') . '<br>' . $i->format('d/m/Y H:i') . '</footer>';
		}

		return $content;
	} 
 
	public function hook_mc_main_javascript() 
	{
		if ( in_array('yes', get_option('bp_manychat_active')) ){
			if (in_array('yes', get_option('bp_manychat_timeractive')) ){
				$current_time = current_time('H:i');
				$open  = get_option('bp_manychat_start_widget'); // "09:30";
				$close = get_option('bp_manychat_stop_widget');  // "17:30";
				if ($this->isBetween($open,$close,$current_time) === TRUE)
				{
				// if widget active with timer
					echo get_option('bp_manychat_textarea_header_js');
				}
			}else{
				// if widget active without timer
				echo get_option('bp_manychat_textarea_header_js');				
			}	
		}		
	}
	
	public function manychat_livechat_widget() 
	{
		if ( in_array('yes', get_option('bp_manychat_active_livechat_widget')) ){
			$current_time = current_time('H:i');
			$open  = get_option('bp_manychat_start_livechat_widget'); // "09:30";
			$close = get_option('bp_manychat_stop_livechat_widget'); // "17:30";
			if ($this->isBetween($open,$close,$current_time) === TRUE)
			{
				echo get_option('bp_manychat_textarea_livechat_widget');
			}
		}
	}
	
}



if( is_admin() ) {
	new Botprophet_ManyChat_Widgets_Backend();
} else {
	new Botprophet_ManyChat_Widgets_Frontend();
}